#
# gateway_cgi.py - Gateway CGI methods.
# Copyright (c) 2005 shinGETsu Project.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: gateway_cgi.py,v 1.24 2005/06/20 04:19:41 fuktommy Exp $
#

import re
import cgi
from time import time

import config
import gateway
from cache import *
from rss import RSS, make_rss1, make_rss2

__version__ = "$Revision: 1.24 $"


class CGI(gateway.CGI):

    """Class for /gateway.cgi."""

    def run(self):
        path = self.path_info()
        form = cgi.FieldStorage(environ=self.environ, fp=self.stdin)

        if path == "new":
            self.header(self.message["new"], deny_robot=True)
            self.print_new_element_form()
            self.footer()
        elif path == "motd":
            self.print_motd()
        elif path == "rss":
            self.print_rss()
        elif form.getfirst("cmd", "") == "new":
            self.jump_new_file(form)
        elif path == "list":
            self.print302(self.list_cgi)
        elif re.search(r"^(list|thread|note)", path):
            m = re.search(r"^(list|thread|note)/?([^/]*)$", path)
            if m is None:
                self.print_title()
                return
            elif m.group(2) != "":
                uri = self.appli[m.group(1)] + self.sep + \
                      self.str_encode(m.group(2))
            elif self.environ.get("QUERY_STRING", "") != "":
                uri = self.appli[m.group(1)] + self.sep + \
                      self.environ["QUERY_STRING"]
            else:
                self.print_title()
                return

            self.print302(uri)
        else:
            self.print_title()

    def print_title(self):
        message = self.message
        self.header(message["logo"] + " - " + message["description"])
        self.stdout.write(
            "<ul>\n" +
            '  <li><em><a href="' + self.list_cgi + '">' +
            message["menu"] + "</a></em></li>\n" +
            '  <li><a href="http://shingetsu.sourceforge.net/">' +
	    message["site"] + "</a>\n" +
            '      (<a href="http://shingetsu.xrea.jp/">' +
            message["mirror"] + "</a>)</li>\n" +
            '  <li><a href="' + self.gateway_cgi + self.sep + 'motd">' +
            message["agreement"] + "</a></li>\n")

        for i in ("index", "update", "status"):
            self.stdout.write('  <li><a href="' + self.admin_cgi + self.sep +
                              i + '">' + message[i] + "</a></li>\n")

        self.stdout.write("</ul>\n" +
                          "<h2>" + message["recent"] + "</h2>\n<ul>\n")

        cachelist = CacheList()
        cachelist.sort(lambda a,b: cmp(b.stamp, a.stamp))
        now = int(time())
        for cache in cachelist:
            if cache.stamp + config.rss_range < now:
                continue
            date = self.localtime(cache.stamp)
            size = cache.size*10/1024/1024/10.0

            self.stdout.write(
                "  <li>\n" +
                "    " + date + "\n" +
                '    <a href="' + self.appli[cache.type] + self.sep +
                self.str_encode(self.file_decode(cache.datfile)) + '">' +
                self.file_decode(cache.datfile) + "</a>\n" +
		"    (" + message[cache.type] + "/" +
                str(len(cache)) + "/" + str(size) +message["mb"] + ")\n" +
                "  </li>\n")
        self.stdout.write("</ul>\n")
        self.footer()


    def jump_new_file(self, form):
        if form.getfirst("link", "") == "":
            self.header(self.message["null_name"], deny_robot=True)
            self.footer()
        elif re.search(r"/|\]\]", form.getfirst("link", "")):
            self.header(self.message["bad_name"], deny_robot=True)
            self.footer()
        elif form.getfirst("type", "") == "":
            self.header(self.message["null_type"], deny_robot=True)
            self.footer()
        elif form.getfirst("type", "") in config.types:
            self.print302(self.appli[form.getfirst("type", "")] + self.sep +
                          self.str_encode(form.getfirst("link", "")))
        else:
            self.print404()

    def print_rss(self):
        rss = RSS(encode = "UTF-8",
                  title = self.message["logo"],
                  parent = "http://" + self.environ["HTTP_HOST"],
                  uri = "http://" + self.environ["HTTP_HOST"]
                                  + self.gateway_cgi + self.sep + "rss",
                  description = self.message["description"])
        cachelist = CacheList()
        now = int(time())
        for cache in cachelist:
            if cache.stamp + config.rss_range >= now:
                title = self.file_decode(cache.datfile)
                rss.append((self.appli[cache.type] + self.sep +
                           self.str_encode(title))[1:],
                           date = cache.stamp,
                           title = title)

        self.stdout.write("Content-Type: text/xml; charset=UTF-8\n")
        try:
            self.stdout.write("Last-Modified: %s\n" %
                              self.rfc822_time(rss[rss.keys()[0]].date))
        except IndexError, KeyError:
            pass
        self.stdout.write("\n")
        if config.rss == "1":
            self.stdout.write(make_rss1(rss))
        else:
            self.stdout.write(make_rss2(rss))

    def print_motd(self):
        self.stdout.write("Content-Type: text/plain\n\n")
        try:
            f = file(config.motd)
            for line in f:
                self.stdout.write(line)
            f.close()
        except IOError:
            self.stderr.write(config.motd + ": IOError\n")
